# Programm 10.7 Test der starken Konvergenz des Milstein-Verfahrens
# am Beispiel dX = r *X*(K -X) dt + peta*X dW, X(0) = Xzero ,
# r = 2 , K= 1 , peta = 1 and Xzero = 0.5 .
#
# Wiener-Prozess-Pfad ueber [0,1], h = 2^(-11).
# Milstein Schritte 128*dt , 64*h , 32*h , 16*h (auch h zum Vergleich)
#
# Test der starken Konvergenz bei T=1: E | X_L - X(T) | 
# vektorisiert, alle Pfade werden simulatan berechnet
randn('state',100) # random seeding
r = 2 ; K = 1 ; peta = 0.25 ; Xzero = 0.5 ; 
T = 1 ; N = 2^(11) ; h = T/N; #
M = 500 ; # Zahl der gesampleten Pfade
R = [ 1 ; 16 ; 32 ; 64 ; 128 ] ; # Milstein-Schritte are R*h
dW = zeros(1,N); # Pre-Allokierung
dW = sqrt(h)*randn(M,N); # Inkremete
Xmil = zeros(M,5) ; # Allokierung
for p = 1:5
   Dt = R(p)*h; L = N/R(p); # L Milstein-Schritte Dt = R h
   Xtemp = Xzero*ones(M,1) ;
   for j = 1:L
     Winc = sum(dW(:,R(p)*(j-1)+1:R(p)*j),2) ;
     Xtemp = Xtemp + Dt*r*Xtemp.*(K-Xtemp) + peta*Xtemp.*Winc...
      + 0.5*peta^2*Xtemp.*(Winc.^2 - Dt);
   endfor
   Xmil(:,p) = Xtemp ; # Milsteinloesung bei t =1
endfor
Xref = Xmil(:,1); # Referenzloesung
Xerr = abs(Xmil(:,2:5) - repmat(Xref,1,4)); # Fehler in den Pfaden
mean(Xerr); # mittlere Pfad-weise Fehler
Dtvals = h*R(2:5); # Milstein Schritte
loglog(Dtvals,mean(Xerr),'b*'), hold on
loglog(Dtvals,Dtvals,'g--') # Referenzsteigung von 1
axis([ 1e-3 1e-1 1e-4 1 ])
xlabel('h','FontSize',16)
ylabel('Mittel von |X(T) - X_L|','FontSize',16)
# Berechnung der Ausgleichskurve der Fehler:  C*h^q 
A = [ones(4,1),log(Dtvals)] ; rhs = log(mean(Xerr)' ) ;
sol = A\rhs ; q = sol(2)
C= exp(sol(1))
loglog(Dtvals,C*Dtvals.^q,'r--'), hold off # tatsaechlich Steigung
resid = norm(A*sol - rhs)
# Programmende
