# Programm 9.1 Loesung der Poisson-Gleichung (pdenr_gb.m)
# auf einen Nichtrechteckgebietmit Dirichlet- und Neumann-RB
# input:  
# output: Loesung u(x,y) auf dem Gitter x,y
# Aufruf: [u,x,y] = pdenr_gb;
function [u,x,y] = pdenr_gb;
# Materialdaten, Quellen
la = 1.0; q = 1.0; ff=0;
# Gebeitslaenge, Gebietshoehe
ll = 1; hh = 10.5*ll/16.0;
ll1 = ll/2.0; hh1 = hh*3.0/7.0;
mm = (hh-hh1)/(ll1-ll);
nn = hh-mm*ll1;
# Dimensionierung des Gitters
nf = 6; n = nf*8+1; m = nf*7+1;
# Gitterinitialisierung
x(1) = 0.0; y(1) = 0.0;
h=1/(nf*8); k=3*h/4;
for i=2:n
  x(i) = (i-1)*h;
endfor
for j=2:m
  y(j) = (j-1)*k;
endfor
# Initialisierung der Gebietsmatrix
for i=1:n
  for j=1:m
    mg(i,j) = 0;
  endfor
endfor
for i=2:n-1
  for j=2:m-1
    mg(i,j) = 1;
  endfor
endfor
for i=1:n-1
  for j=1:m-1
    if (j == m-1)
      mg(i,j) = 3;
    endif
    if (i == 1)
      mg(i,j) = 5;
    endif
    if (j == 1)
      mg(i,j) = 7;
    endif
    if (i == n-1)
      mg(i,j) = 9;
    endif
  endfor
endfor
mg(1,m-1) = 4; mg(1,1) = 6; mg(n-1,1) = 8;
for i=1:n-1
  for j=1:m-1
    if (abs(mm*x(i)+nn-y(j)) < 10^(-6))
      mg(i,j) = 2;
    endif
    if (y(j) > mm*x(i)+nn)
      mg(i,j) = 0;
    endif
  endfor
endfor
# Startfeld incl. homogener Dirichlet-Randbedingungen
for i=1:n
  for j=1:m
    u(i,j) = 0.0;
  endfor
endfor 
# Konstanten
omega = 1.6;
h2=h^2; k2=k^2; 
dd1=1/(2/h2+2/k2); dd2=1/(1/h2+1/k2); dd3=1/(2/h2+3/k2); dd4=1/(3/h2+3/k2);
dd5=1/(3/h2+2/k2); dd6=dd4; dd7=dd3; dd8=dd4; dd9=dd5;
whk=q*sqrt(h2+k2)/(h*k)/la; udiff = 1; unorm = 1; it = 0; la2=2*la;
# Randwerte
un=0; us=0; uo=0; uw=0;
# sor-Iteration
while (sqrt(udiff/unorm) > 10^(-3) && it < 1200)
  ualt = u;
  for i=1:n-1
    for j=1:m-1
      if (mg(i,j) == 1)
        u(i,j) = dd1*(ff/la + (u(i+1,j) + u(i-1,j))/h2 + (u(i,j+1)+u(i,j-1))/k2);
      endif
      if (mg(i,j) == 2)
        u(i,j) = dd2*(ff/la2 + u(i-1,j)/h2 + u(i,j-1)/k2 + whk);
      endif
      if (mg(i,j) == 3)
        u(i,j) = dd3*(ff/la + (u(i+1,j) + u(i-1,j))/h2 + (2*un+u(i,j-1))/k2);
      endif
      if (mg(i,j) == 4)
        u(i,j) = dd4*(ff/la + (u(i+1,j) + 2*uw)/h2 + (2*un+u(i,j-1))/k2);
      endif
      if (mg(i,j) == 5)
        u(i,j) = dd5*(ff/la + (u(i+1,j) + 2*uw)/h2 + (u(i,j+1)+u(i,j-1))/k2);
      endif
      if (mg(i,j) == 6)
        u(i,j) = dd6*(ff/la + (u(i+1,j) + 2*uw)/h2 + (u(i,j+1)+2*us)/k2);
      endif
      if (mg(i,j) == 7)
        u(i,j) = dd7*(ff/la + (u(i+1,j) + u(i-1,j))/h2 + (u(i,j+1)+2*us)/k2);
      endif
      if (mg(i,j) == 8)
        u(i,j) = dd8*(ff/la + (2*uo + u(i-1,j))/h2 + (u(i,j+1)+2*us)/k2);
      endif
      if (mg(i,j) == 9)
        u(i,j) = dd9*(ff/la + (2*uo + u(i-1,j))/h2 + (u(i,j+1)+u(i,j-1))/k2);
      endif
    endfor
  endfor
  udiff = 0.0; unorm = 0.0;
  for i=1:n-1
    for j=1:m-1
      u(i,j) = omega*u(i,j)+(1-omega)*ualt(i,j);
      udiff = udiff + (u(i,j)-ualt(i,j))^2;
      unorm = unorm + ualt(i,j)^2;
    endfor
  endfor
  if (unorm == 0.0)
    unorm = 1;
  endif
  it = it + 1;
endwhile
endfunction
